class Seexpr < Formula
  desc "Embeddable expression evaluation engine"
  homepage "https://wdas.github.io/SeExpr/"
  url "https://github.com/wdas/SeExpr/archive/v3.0.1.tar.gz"
  sha256 "1e4cd35e6d63bd3443e1bffe723dbae91334c2c94a84cc590ea8f1886f96f84e"
  license "Apache-2.0"

  livecheck do
    url :stable
    regex(/^v?(\d+(?:\.\d+)+)$/i)
  end

  depends_on "cmake" => :build
  depends_on "doxygen" => :build
  depends_on "libpng"

  def install
    mkdir "build" do
      system "cmake", "..", *std_cmake_args, "-DUSE_PYTHON=FALSE"
      system "make", "doc"
      system "make", "install"
    end
  end

  test do
    actual_output = shell_output("#{bin}/asciigraph2").lines.map(&:rstrip).join("\n")
    expected_output = <<~EOS
                                    |
                                    |
                                    |
                                    |
                                   ###
                                  # |#
                                 ## |##
                                 #  | #
                                ##  | ##
                                #   |  #
                               ##   |  ##
                               #    |   #
                               #    |   ##
                   ####       #     |    #       ####
      #######-----##--###-----#-----|----##-----##--###-----######
            ######      ##   #      |     #    #      ######
                         ## ##      |     ## ##
                          ###       |      ###
                                    |
                                    |
                                    |
                                    |
                                    |
                                    |
                                    |
                                    |
                                    |
                                    |
                                    |
                                    |
    EOS

    assert_equal actual_output, expected_output.rstrip
  end
end
